@@ -138,7 +138,8 @@ on_window1_button_press_event          (GtkWidget       *widget,
                                         gpointer         user_data)
 {
         GokKey *pKey = (GokKey *) user_data;
-	if (event->button == 1)
+	if (event->button == 1 && 
+	    !gok_scanner_current_state_uses_core_mouse_button (1))
 	{
 	        gok_feedback_set_selected_key (pKey);
 		gok_keyboard_output_selectedkey ();
