@@ -1163,7 +1163,11 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 * gok_keylabel_new
 * @pKey: Pointer to the key that gets the new label.
 * @pLabelText: Text string for this label.
-* @pModifierText: Modifier text string (e.g. "shift" or "ctrl").
+* @level: the level (see XKB spec) for which this label is valid.
+* @group: the group (see XKB spec) for which this label is valid.
+* @vmods: a delimited list of virtual modifier names which must be matched 
+*         in order for this label to be valid, or NULL if no virtual 
+*         modifiers are relevant to this label.
 *
 * Allocates memory for a new key label and initializes the GokKeyLabel structure.
 * Returns a pointer to the new key label, NULL if it can't be created.
@@ -1527,27 +1531,42 @@ gchar* gok_key_get_label (GokKey* pKey)
 	guint level;
 	guint group = gok_key_get_effective_group ();
 	guint mods = gok_spy_get_modmask ();
-	XkbDescRec *xkb;
-
-	xkb = gok_keyboard_get_xkb_desc (),
-	keycode = gok_output_get_keycode (pKey->pOutput);
-	type = gok_key_get_xkb_type_index (xkb, keycode, group);
+	XkbDescRec *xkb = gok_keyboard_get_xkb_desc ();
+	GokOutputType output_type = OUTPUT_INVALID;
 
 #ifdef GOK_DEBUG
 	char *typename = XGetAtomName (gok_keyboard_get_display (), xkb->map->types[type].name);
-	fprintf (stderr, "keycode %d, group %d, type=%d [%s]\n", keycode, group, type, typename);
 #endif
-	level = gok_key_level_for_type (gok_keyboard_get_display (), 
-					xkb,
-					type, &mods);
 
+	if (pKey->pOutput) output_type = pKey->pOutput->Type;
+	switch (output_type)
+	{
+	    case OUTPUT_KEYCODE:
+	    case OUTPUT_KEYSYM:
+		keycode = gok_output_get_keycode (pKey->pOutput);
+		type = gok_key_get_xkb_type_index (xkb, keycode, group);
+		
 #ifdef GOK_DEBUG
-	if (pKey->pLabel) fprintf (stderr, "Key %s: type %d, level %d [mods %x, preserved %x]\n", 
-				pKey->pLabel->Text,
-				type, level, gok_spy_get_modmask (), mods);
+		fprintf (stderr, "keycode %d, group %d, type=%d [%s]\n", keycode, group, type, typename);
 #endif
+		level = gok_key_level_for_type (gok_keyboard_get_display (), 
+						xkb,
+						type, &mods);
+#ifdef GOK_DEBUG
+		if (pKey->pLabel) fprintf (stderr, "Key %s: type %d, level %d [mods %x, preserved %x]\n", 
+					   pKey->pLabel->Text,
+					   type, level, gok_spy_get_modmask (), mods);
+#endif
+		break;
+	    case OUTPUT_KEYSTRING:
+		level = gok_key_level_for_type (gok_keyboard_get_display (), xkb, XkbAlphabeticIndex, &mods);
+		break;
+	    default:
+		break;
+	}
+		
 	g_assert (pKey != NULL);
-	
+		
 	pTextReturned = m_TextEmpty;
 	
 	/* get the 'normal' label */
