@@ -133,7 +133,7 @@ typedef struct GokKeyboard {
 	struct GokChunk* pChunkFirst;
 } GokKeyboard;
 
-int gok_xkb_base_event_type;
+extern int gok_xkb_base_event_type;
 
 GokKeyboard* gok_keyboard_new (void);
 GokKeyboard* gok_keyboard_read (gchar* Filename);
