@@ -63,7 +63,7 @@ gboolean gok_page_actions_initialize (GtkWidget* pWindowSettings)
 	GList* items = NULL;
 	GSList* pDevice = NULL;
 	GSList* slist;
-	gboolean bReturnCode;
+	gboolean bReturnCode = TRUE; /* unused? */
 	
 	g_assert (pWindowSettings != NULL);
 	m_pWindowSettings = pWindowSettings;
