@@ -36,10 +36,17 @@ static GtkWidget* m_pWindowSettings;
 /* settings for the page */
 static gboolean m_WordCompleteOnOff;
 static int m_NumberPredictions;
+static gboolean m_bUseAuxDicts = FALSE;
+static gchar *m_AuxDicts = NULL;
 
 /* backup of the page data */
 static gboolean m_WordCompleteOnOffBackup;
 static int m_NumberPredictionsBackup;
+static gboolean m_bUseAuxDictsBackup = FALSE;
+static gchar *m_AuxDictsBackup = NULL;
+
+/* privates */
+void gok_page_wordcomplete_initialize_auxwordlist_control(const char* file);
 
 /**
 * gok-page-wordcomplete-initialize
@@ -52,26 +59,34 @@ static int m_NumberPredictionsBackup;
 **/
 gboolean gok_page_wordcomplete_initialize (GtkWidget* pWindowSettings)
 {
-	GtkWidget* pCheckboxOnOff;
-	GtkWidget* pSpinNumberPredictions;
+	GtkWidget* pWidget;
 	
 	g_assert (pWindowSettings != NULL);
 		
 	m_pWindowSettings = pWindowSettings;
 	
-	/* store the current values */
 	m_WordCompleteOnOff = gok_data_get_wordcomplete();
 	m_NumberPredictions = gok_data_get_num_predictions();
-	gok_page_wordcomplete_backup();
+	m_bUseAuxDicts = gok_data_get_use_aux_dictionaries();
+	m_AuxDicts = g_strdup (gok_data_get_aux_dictionaries());
 	
-	/* update the controls */
-	pCheckboxOnOff = lookup_widget (m_pWindowSettings, "checkWordCompletion");
-	g_assert (pCheckboxOnOff != NULL);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pCheckboxOnOff), gok_data_get_wordcomplete());
+	/* store the current values */
+	gok_page_wordcomplete_backup();
 
-	pSpinNumberPredictions = lookup_widget (m_pWindowSettings, "spinNumberPredictions");
-	g_assert (pSpinNumberPredictions != NULL);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON(pSpinNumberPredictions), gok_data_get_num_predictions());
+	/* update the controls */
+	pWidget = lookup_widget (m_pWindowSettings, "checkWordCompletion");
+	g_assert (pWidget != NULL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pWidget), gok_data_get_wordcomplete());
+
+	pWidget = lookup_widget (m_pWindowSettings, "spinNumberPredictions");
+	g_assert (pWidget != NULL);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON(pWidget), gok_data_get_num_predictions());
+	
+	pWidget = lookup_widget (m_pWindowSettings, "checkExtraWordList");
+	g_assert (pWidget != NULL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pWidget), gok_data_get_use_aux_dictionaries());
+
+	gok_page_wordcomplete_initialize_auxwordlist_control((const char*)m_AuxDicts);
 	
 	return TRUE;
 }
@@ -95,22 +110,24 @@ void gok_page_wordcomplete_refresh ()
 **/
 gboolean gok_page_wordcomplete_apply ()
 {
-	GtkWidget* pCheckboxOnOff;
-	GtkWidget* pSpinNumberPredictions;
+	GtkWidget* pWidget;
 	gboolean bDataChanged;
-
+	gchar* text;
+	
 	g_assert (m_pWindowSettings != NULL);
 	
 	bDataChanged = FALSE;
 
-	pCheckboxOnOff = lookup_widget (m_pWindowSettings, "checkWordCompletion");
-	g_assert (pCheckboxOnOff != NULL);
-	m_WordCompleteOnOff = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pCheckboxOnOff));
+	pWidget = lookup_widget (m_pWindowSettings, "checkWordCompletion");
+	g_assert (pWidget != NULL);
+	m_WordCompleteOnOff = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pWidget));
 
-	pSpinNumberPredictions = lookup_widget (m_pWindowSettings, "spinNumberPredictions");
-	g_assert (pSpinNumberPredictions != NULL);
-	m_NumberPredictions = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(pSpinNumberPredictions));
+	pWidget = lookup_widget (m_pWindowSettings, "spinNumberPredictions");
+	g_assert (pWidget != NULL);
+	m_NumberPredictions = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(pWidget));
 
+	gok_page_wordcomplete_update_auxwordlist_from_control();
+	
 	/* update the gok data with any new settings */
 	if (gok_data_get_wordcomplete() != m_WordCompleteOnOff)
 	{
@@ -126,6 +143,12 @@ gboolean gok_page_wordcomplete_apply ()
 		gok_keyslotter_change_number_predictions (m_NumberPredictions, KEYTYPE_WORDCOMPLETE);
 	}
 
+	if (gok_data_get_use_aux_dictionaries() != m_bUseAuxDicts) {
+		bDataChanged = TRUE;
+		gok_data_set_use_aux_dictionaries ( m_bUseAuxDicts );
+		/* TODO call rebuild internal word completion list */
+	}
+	
 	return bDataChanged;
 }
 
@@ -139,8 +162,7 @@ gboolean gok_page_wordcomplete_apply ()
 **/
 gboolean gok_page_wordcomplete_revert ()
 {
-	GtkWidget* pCheckboxOnOff;
-	GtkWidget* pSpinNumberPredictions;
+	GtkWidget* pWidget;
 	gboolean bDataChanged;
 
 	bDataChanged = FALSE;
@@ -153,9 +175,9 @@ gboolean gok_page_wordcomplete_revert ()
 		gok_data_set_wordcomplete (m_WordCompleteOnOff);
 		gok_keyslotter_on (m_WordCompleteOnOff, KEYTYPE_WORDCOMPLETE);
 
-		pCheckboxOnOff = lookup_widget (m_pWindowSettings, "checkWordCompletion");
-		g_assert (pCheckboxOnOff != NULL);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pCheckboxOnOff), m_WordCompleteOnOff);
+		pWidget = lookup_widget (m_pWindowSettings, "checkWordCompletion");
+		g_assert (pWidget != NULL);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pWidget), m_WordCompleteOnOff);
 
 	}
 
@@ -166,9 +188,35 @@ gboolean gok_page_wordcomplete_revert ()
 		gok_data_set_num_predictions (m_NumberPredictions);
 		gok_keyslotter_change_number_predictions (m_NumberPredictions, KEYTYPE_WORDCOMPLETE);
 
-		pSpinNumberPredictions = lookup_widget (m_pWindowSettings, "spinNumberPredictions");
-		g_assert (pSpinNumberPredictions != NULL);
-		gtk_spin_button_set_value (GTK_SPIN_BUTTON(pSpinNumberPredictions), m_NumberPredictions);
+		pWidget = lookup_widget (m_pWindowSettings, "spinNumberPredictions");
+		g_assert (pWidget != NULL);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON(pWidget), m_NumberPredictions);
+	}
+	
+	if (m_bUseAuxDicts != m_bUseAuxDictsBackup) {
+		bDataChanged = TRUE;
+		m_bUseAuxDicts = m_bUseAuxDictsBackup;
+		gok_data_set_use_aux_dictionaries (m_bUseAuxDicts);
+		/* TODO call rebuild internal dictionary */
+		gok_log ("TODO: Call to rebuild word completion model");
+
+		pWidget = lookup_widget (m_pWindowSettings, "checkExtraWordList");
+		g_assert (pWidget != NULL);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pWidget), m_bUseAuxDicts);
+	}
+	
+	if (m_AuxDicts != m_AuxDictsBackup) {
+		bDataChanged = TRUE;
+		g_free(m_AuxDicts);
+		m_AuxDicts = g_strdup(m_AuxDictsBackup);
+		gok_data_set_aux_dictionaries (m_AuxDicts);
+		/* TODO call rebuild internal word completion list */
+		gok_log ("TODO: Call to rebuild word completion model");
+		
+		pWidget = lookup_widget (m_pWindowSettings, "fileentry1");
+		g_assert (pWidget != NULL);
+		gnome_file_entry_set_filename ((GnomeFileEntry*)pWidget,m_AuxDicts); 
+		
 	}
 
 	return bDataChanged;
@@ -183,6 +231,52 @@ void gok_page_wordcomplete_backup ()
 {
 	m_WordCompleteOnOffBackup = m_WordCompleteOnOff;
 	m_NumberPredictionsBackup = m_NumberPredictions;
+	m_bUseAuxDictsBackup = m_bUseAuxDicts;
+	m_AuxDictsBackup = g_strdup(m_AuxDicts);
+}
+
+
+void 
+gok_page_wordcomplete_toggle_wordlist (gboolean on)
+{
+	gok_log ("Use aux wordlist toggled");	
+	m_bUseAuxDicts = on;
+	/* TODO: enable/disable controls, but wait until 
+	   implemented this for other cases too (for consistency)*/
+}
+
+void 
+gok_page_wordcomplete_update_auxwordlist_from_control ()
+{
+	GtkWidget* pWidget; 
+	gchar* text;
+	
+	gok_log_enter();
+	pWidget = lookup_widget (m_pWindowSettings, "fileentry1");
+	g_assert (pWidget != NULL);
+	text = gnome_file_entry_get_full_path ( (GnomeFileEntry*)pWidget, FALSE);
+	/* TODO call format check on this word list */
+	/* TODO call rebuild internal word completion list */
+	gok_log_x ("TODO: Call to check file for expected format on file [%s]", text);
+	
+	g_free (m_AuxDicts);
+	m_AuxDicts = text;
+	gok_log_leave();
+}
+
+void 
+gok_page_wordcomplete_initialize_auxwordlist_control (const char* file)
+{
+	GtkWidget* pWidget; 
+	
+	gok_log_enter();
+	pWidget = lookup_widget (m_pWindowSettings, "fileentry1");
+	g_assert (pWidget != NULL);
+	gnome_file_entry_set_modal ((GnomeFileEntry*)pWidget, FALSE);
+	gnome_file_entry_set_title ((GnomeFileEntry*)pWidget, _("Select extra word list file"));
+	gnome_file_entry_set_directory_entry ((GnomeFileEntry*)pWidget, FALSE);
+	gnome_file_entry_set_filename ( (GnomeFileEntry*)pWidget, file);
+	gok_log_leave();
 }
 
 
