@@ -39,21 +39,13 @@
 #include "support.h"
 #include "gok-log.h"
 #include "main.h"
+#include "gok-gconf-keys.h"
 
 /* pointer to the settings dialog window */
 static GtkWidget* m_pWindowSettings;
 
 static gboolean is_locked;
 
-void
-gok_settingsdialog_lock (void)
-{
-	is_locked = TRUE;
-}
-
-
-/* Private functions */
-
 /**
 * gok_settingsdialog_open
 * 
@@ -73,6 +65,7 @@ gboolean gok_settingsdialog_open (gboolean bShow)
 	GtkWidget* pButtonHelp;
 	GtkWidget* hbuttonbox;
 	GtkTooltips *tooltips;
+	gchar *deprecated_string;
 
 	m_pWindowSettings = create_dialogSettings();
 	if (m_pWindowSettings == NULL)
@@ -81,6 +74,8 @@ gboolean gok_settingsdialog_open (gboolean bShow)
 		return FALSE;
 	}
 
+	is_locked = FALSE;  /* don't read gconf here */
+
 	/* initialize all the pages */
 	gok_settings_page_keysizespace_initialize (m_pWindowSettings);
 	gok_page_accessmethod_initialize (m_pWindowSettings);
@@ -124,7 +119,7 @@ gboolean gok_settingsdialog_open (gboolean bShow)
 #ifndef EARLY_GTK_VERSION
 /* these calls can be used only on GTK 1.3 and above */
 
-	pButtonTry = gtk_button_new_from_stock (GTK_STOCK_JUMP_TO);
+	pButtonTry = gtk_button_new_from_stock (GTK_STOCK_APPLY);
 	pButtonRevert = gtk_button_new_from_stock (GTK_STOCK_REVERT_TO_SAVED);
 	pButtonOK = gtk_button_new_from_stock (GTK_STOCK_OK);
 	pButtonCancel = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
@@ -146,8 +141,7 @@ gboolean gok_settingsdialog_open (gboolean bShow)
                       GTK_SIGNAL_FUNC (on_button_help),
                       NULL);
 	
-	gtk_button_set_label (GTK_BUTTON(pButtonTry), "_Try");
-/*	gtk_button_set_use_stock (GTK_BUTTON(pButtonTry), TRUE); */
+	gtk_button_set_use_stock (GTK_BUTTON(pButtonTry), TRUE); 
 
 	gtk_widget_ref (pButtonTry);
 	gtk_object_set_data_full (GTK_OBJECT (m_pWindowSettings), "pButtonTry", pButtonTry,
@@ -155,12 +149,15 @@ gboolean gok_settingsdialog_open (gboolean bShow)
 	gtk_widget_show (pButtonTry);
 	gtk_box_pack_start (GTK_BOX (hbuttonbox), pButtonTry, FALSE, FALSE, 10);
 	GTK_WIDGET_SET_FLAGS (pButtonTry, GTK_CAN_DEFAULT);
-	gtk_tooltips_set_tip (tooltips, pButtonTry, _("try these settings"), NULL);
+
+	/* this string is deprecated and will be removed in the next release. */
+	deprecated_string = _("_Try");
+	/* this string is deprecated and will be removed in the next release. */
+	deprecated_string = _("try these settings");
 
 	gtk_signal_connect (GTK_OBJECT (pButtonTry), "clicked",
                       GTK_SIGNAL_FUNC (on_button_try),
                       NULL);
-
 	gtk_widget_ref (pButtonRevert);
 	gtk_object_set_data_full (GTK_OBJECT (m_pWindowSettings), "pButtonRevert", pButtonRevert,
 	                            (GtkDestroyNotify) gtk_widget_unref);
@@ -200,10 +197,9 @@ gboolean gok_settingsdialog_open (gboolean bShow)
                       NULL);
 
 	
-	/* show/hide the settings dialog */
 	if (bShow == TRUE)
 	{
-		gtk_widget_show (m_pWindowSettings);
+		gok_settingsdialog_show ();
 	}
 
 	return TRUE;
@@ -220,19 +216,30 @@ gboolean gok_settingsdialog_show()
 {
 	gboolean bFirstShowing;
 	GtkWidget *label;
+	GtkWidget *notebook;
+	gboolean was_locked;
 
 	g_assert (m_pWindowSettings != NULL);
 
-	bFirstShowing = (GDK_IS_DRAWABLE(m_pWindowSettings->window) == FALSE) ? TRUE : FALSE;
-
-	/* if we've "locked" certain prefs, de-sensitize the relevant page tabs */
-	if (is_locked) {
-		label = lookup_widget (m_pWindowSettings, "label2");
-		gtk_widget_set_sensitive (label, FALSE);
-		label = lookup_widget (m_pWindowSettings, "label4");
-		gtk_widget_set_sensitive (label, FALSE);
+	was_locked = is_locked;
+	gok_gconf_get_bool (gconf_client_get_default(),
+		GOK_GCONF_PREFS_LOCKED, &is_locked);
+	
+	if (was_locked != is_locked) {
+		notebook = lookup_widget (m_pWindowSettings, "notebook1");
+		/* FIXME: magic #'s used - easily fixed later when we libglade-ify */
+		if (!is_locked) {
+			gtk_widget_show(gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1));
+			gtk_widget_show(gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 3));
+		}
+		else {
+			gtk_widget_hide(gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1));
+			gtk_widget_hide(gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 3));
+		}
 	}
 
+	bFirstShowing = (GDK_IS_DRAWABLE(m_pWindowSettings->window) == FALSE) ? TRUE : FALSE;
+
 	gtk_widget_show (m_pWindowSettings);
 
 	/* if this is the first time we're showing the dialog then redisplay
@@ -276,16 +283,17 @@ GtkWidget* gok_settingsdialog_get_window ()
 **/
 void gok_settingsdialog_close()
 {
+	gok_log_enter();
 	if (m_pWindowSettings != NULL)
 	{
-		gtk_widget_destroy (m_pWindowSettings);
-	
 		gok_page_accessmethod_close();
+		gtk_widget_destroy (m_pWindowSettings);
 	}
 	else
 	{
 		g_warning("The settings dialog doesn't exist in close function!");
 	}
+	gok_log_leave();
 }
 
 /**
