@@ -563,6 +563,7 @@ gok_spy_node_match (AccessibleNode *node, GokSpySearchType type)
 	case GOK_SPY_SEARCH_CHILDREN:
 	case GOK_SPY_SEARCH_COMBO:
 	case GOK_SPY_SEARCH_LISTITEMS:
+	case GOK_SPY_SEARCH_TABLE_ROWS:
 	case GOK_SPY_SEARCH_ALL:
 		return TRUE;
 		break;
@@ -647,7 +648,6 @@ Accessible *
 gok_spy_get_list_parent (Accessible *search_root)
 {
     Accessible *child = NULL;
-
     gok_spy_accessible_ref (search_root);
     while (search_root) 
     {
@@ -681,6 +681,136 @@ gok_spy_get_list_parent (Accessible *search_root)
     return NULL;
 }
 
+static gchar*
+gok_spy_concatenate_child_names (Accessible *parent)
+{
+	gint num_children = 0;
+	gchar *tmp = NULL, *name = NULL;
+	if (parent) 
+	{
+		gint i;
+		num_children = Accessible_getChildCount (parent);
+		for (i = 0; i < num_children; ++i) 
+		{
+			Accessible *child = Accessible_getChildAtIndex (parent, i);
+			if (child ) 
+			{
+				gchar *childname = Accessible_getName (child);
+				if (childname && (strlen (childname) > 0)) 
+				{
+					if (tmp)
+					{
+						name = g_strconcat (tmp, "", childname, NULL);
+						g_free (tmp);
+					}
+					else
+					{
+						name = g_strdup (childname);
+					}
+					SPI_freeString (childname);
+					tmp = name;
+				}
+			}
+		}
+	}
+	return name;
+}
+
+/**
+ * 
+ * gok_spy_get_table_row_nodes:
+ * Returns a list of nodes which are children of a table, such that
+ * there is exactly one child per table row plus one child per actionable table header.
+ *
+ **/
+GSList *
+gok_spy_get_table_row_nodes (Accessible *search_root)
+{
+	GSList *nodes = NULL;
+	AccessibleNodeFlags flags;
+	AccessibleTable *table;
+	Accessible *child, *header, *first_child, *last_child;
+	AccessibleComponent *component;
+	long row_count, col_count;
+	long i, first_row, last_row, first_index = 0, last_index = -1;
+	long x, y, width, height;
+
+	g_assert (Accessible_isTable (search_root));
+	table = Accessible_getTable (search_root);
+	flags.value = 0xff;
+	col_count = AccessibleTable_getNColumns (table);
+	if (col_count > 1)
+	{
+		for (i = 0; i < col_count; ++i)
+		{
+			header = AccessibleTable_getColumnHeader (table, i);
+			if (header)
+			{
+				gok_spy_accessible_implicit_ref (header);
+				nodes = gok_spy_append_node (nodes, header, flags);
+				gok_spy_accessible_unref (header);
+			}
+		}
+	}
+
+	component = Accessible_getComponent (table);
+	if (!component) return NULL;
+
+	AccessibleComponent_getExtents (component, &x, &y, &width, &height, SPI_COORD_TYPE_SCREEN);
+	first_child = AccessibleComponent_getAccessibleAtPoint (component, x, y, SPI_COORD_TYPE_SCREEN);
+	last_child = AccessibleComponent_getAccessibleAtPoint (component, x + width, y + height, SPI_COORD_TYPE_SCREEN);
+	AccessibleComponent_unref (component);
+
+	if (first_child) 
+		first_index = Accessible_getIndexInParent (first_child);
+	if (last_child) 
+		last_index = Accessible_getIndexInParent (last_child);
+	Accessible_unref (first_child);
+	Accessible_unref (last_child);
+
+	row_count = AccessibleTable_getNRows (table);
+	first_row = AccessibleTable_getRowAtIndex (table, first_index);
+	if (last_index >= 0) 
+		last_row = AccessibleTable_getRowAtIndex (table, last_index);
+	else 
+		last_row = row_count - 1;
+
+	last_row = MIN (last_row, row_count - 1);
+	first_row = MAX (first_row, 0);
+	
+	gok_log ("adding nodes for rows %d through %d (inclusive)\n", first_row, last_row);
+
+	for (i = first_row; i <= last_row; ++i)
+	{
+		gint n = 0;
+		/* make sure the row cell we choose is not anonymous */
+		/* assignment operator on child ("=" vs "==") is intentional */
+		while ((child = AccessibleTable_getAccessibleAt (table, i, n)) && (n < col_count))
+		{
+			char *name = Accessible_getName (child);
+			char *desc = Accessible_getDescription (child);
+			gboolean has_name = (name && (strlen (name) > 0)), has_desc = (desc && (strlen (desc) > 0));
+
+			gok_spy_accessible_implicit_ref (child);
+			if (name) SPI_freeString (name);
+			if (desc) SPI_freeString (desc);
+			/* we make exception for compound cells, which will scavenge a name from their children */
+			if (has_name || has_desc || (Accessible_getChildCount (child) > 0))
+			{
+				nodes = gok_spy_append_node (nodes, child, flags);
+				gok_spy_accessible_unref (child);
+				break;
+			}
+			else 
+			{
+				gok_spy_accessible_unref (child);
+			}
+			++n;
+		}
+	}
+	return nodes;
+}
+
 /**
  * 
  * gok_spy_get_children:
@@ -1069,6 +1199,7 @@ gok_spy_is_ui (Accessible *accessible, AccessibleRole role)
 	default:
 		if (Accessible_isEditableText (accessible) ||
 		    Accessible_isHypertext (accessible) ||
+		    Accessible_isTable (accessible) ||
 		    (Accessible_isAction (accessible) && !gok_spy_is_menu_role (role)))
 			interesting = TRUE;
 		gok_log ("checking for interesting interfaces... %s\n", interesting ? "yes" : "no");
@@ -1401,7 +1532,20 @@ GSList* gok_spy_append_node (GSList* nodes,
 			}
 		}
 
-		if (pName == NULL) { /* desperate attempt to reuse singleton-child's name */
+		/* 
+		 * Role "TABLE-CELL" is used for 'generic' table children. 
+		 * These children's behavior is governed more by the fact that they are
+		 * children of a table than by their class-specific characteristics (unlike,
+		 * for instance, ROLE_CHECKBUTTON or ROLE_PUSH_BUTTON objects which may be
+		 * stored in tables).  Some table cells are composite objects, and we should
+		 * attempt to create a name string from the concatenation of their children 
+		 * when possible.  In the case of GtkTreeviews in particular, where row-selection
+		 * is the dominant user activity, this is especially appropriate.
+		 */
+		if (pName == NULL && (Accessible_getRole (pAccessible) == SPI_ROLE_TABLE_CELL)) {
+			pName = gok_spy_concatenate_child_names (pAccessible);
+		}
+		else if (pName == NULL) { /* desperate attempt to reuse singleton-child's name */
 			Accessible *tmp_parent = pAccessible;
 			gok_log("no description... checking for singleton child with name.");
 			maxloops = 3;
@@ -1432,6 +1576,49 @@ GSList* gok_spy_append_node (GSList* nodes,
 				--maxloops;
 			}
 		}
+		if (pName == NULL && Accessible_isTable (pAccessible)) {
+		        /* Tables are so important we must include them somehow */
+			AccessibleTable *table = Accessible_getTable (pAccessible);
+			Accessible *caption = AccessibleTable_getCaption (table);
+			if (caption) 
+			{
+				char *caption_string = Accessible_getName (caption);
+				if (caption_string) 
+				{
+					pName = g_strndup (caption_string, 20);
+					SPI_freeString (caption_string);
+				}
+			}
+			if (pName == NULL && AccessibleTable_getNColumns (table) == 1) 
+			{
+				char *desc = AccessibleTable_getColumnDescription (table, 0);
+				if (desc) 
+				{
+					pName = g_strndup (desc, 20);
+					SPI_freeString (desc);
+				}
+				else
+				{
+					Accessible *header;
+					header = AccessibleTable_getColumnHeader (table, 0);
+					if (header) 
+					{
+						desc = Accessible_getName (header);
+						if (desc) 
+						{
+							pName = g_strndup (desc, 20);
+							SPI_freeString (desc);
+						}
+						Accessible_unref (header);
+					}
+				}
+			}
+			if (pName == NULL)
+			{
+				/* TODO: I18N mark this string for translation! */
+				pName = g_strdup ("Table");
+			}
+		}			
 
 		if (pName != NULL) {
 			gok_log("we have a name: [%s]",pName);
@@ -1506,7 +1693,6 @@ GSList* gok_spy_append_node (GSList* nodes,
 				nodes = gok_spy_add_node (nodes, pAccessible, flags, 0, pName);
 			}
 		}
-			
 	}
 	
 	gok_log_leave();
@@ -2038,8 +2224,13 @@ gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboar
 					_priv_ui_nodes = gok_spy_append_node (_priv_ui_nodes, toolbar_item, flags);
 					Accessible_unref (toolbar_item);
 				}
+				else
+				{
+				    ui_flags.value |= gok_spy_search_component_list (child, keyboard_ui_flags).value;
+				}
 			}
-			else if ((keyboard_ui_flags.data.gui || keyboard_ui_flags.data.editable_text) && !flags.data.is_menu && gok_spy_is_ui (child, role))
+			if ((keyboard_ui_flags.data.gui || keyboard_ui_flags.data.editable_text) && 
+			    !flags.data.is_menu && !flags.data.is_toolbar_item && gok_spy_is_ui (child, role))
 			{
 				if (keyboard_ui_flags.data.gui) 
 				{
