@@ -367,7 +367,8 @@ gok_wordcomplete_unichar_is_compose (GokWordComplete *complete, gunichar letter)
 static gboolean
 gok_wordcomplete_unichar_is_combine (GokWordComplete *complete, gunichar letter)
 {
-	return (g_unichar_type (letter) == G_UNICODE_COMBINING_MARK);
+	GUnicodeType utype = g_unichar_type (letter);
+	return (utype == G_UNICODE_COMBINING_MARK || utype == G_UNICODE_NON_SPACING_MARK);
 }
 
 static gboolean 
@@ -504,7 +505,7 @@ gok_wordcomplete_last_combine (GokWordComplete *complete, gunichar combining_let
 		gchar utf8char[7];
 		gchar *tmp, *tmp2;
 		utf8char[g_unichar_to_utf8 (combining_letter, utf8char)] = '\0'; /* null terminate utf8 string */
-		tmp = g_strconcat (word, utf8char);
+		tmp = g_strconcat (word, utf8char, NULL);
 		if (tmp)
 		{
 			gint len;
@@ -620,6 +621,7 @@ gok_wordcomplete_real_process_unichar (GokWordComplete *complete, gunichar lette
 		prefix = gok_wordcomplete_unichar_push (complete, letter);
 		break;
 	}
+
 	return prefix;
 }
 
